@@ -22,9 +22,16 @@ def parse_args():
     # parser.add_argument('-h', '--help', action='store_true')
     parser.add_argument("-v", "--version", action="store_true")
     # parser.add_argument('-d', '--debug', action='store_true')
-    # parser.add_argument('-q', '--quiet', action='store_true')
+    parser.add_argument(
+        '-q',
+        '--quiet',
+        help="operate in complete silence",
+        action='store_true'
+    )
     # parser.add_argument('-c', '--config', action='store')
     # parser.add_argument('-i', '--input', action='store_true') # create trees from terminal input
+    # parser.add_argument('-r', '--remove', action='store_true') # remove the tree file after creation
+    # parser.add_argument('-l', '--label', action='store_true') # only run the trees in the file that have a given label
     parser.add_argument(
         "-f",
         "--file",
@@ -60,6 +67,9 @@ def parse_args():
 
 
 def do_things(arguments):
+
+    _is_quiet = arguments.quiet
+
     if arguments.version is True:
         from sharpshooter import __version__
         print(__version__)
@@ -69,7 +79,7 @@ def do_things(arguments):
         with open(arguments.test.name, "r") as f:
             filecontent = f.read()
             f.close()
-        tree(filecontent, test=True)
+        tree(filecontent, test=True, quiet=_is_quiet)
         return
     if arguments.file is not None:
         # TODO - if no file passed find one called .tree and use it anyway? at least ask for confirmation
@@ -77,7 +87,7 @@ def do_things(arguments):
         with open(arguments.file.name, "r") as f:
             filecontent = f.read()
             f.close()
-        tree(filecontent)
+        tree(filecontent, quiet=_is_quiet)
         return
 
     if arguments.jinja is not None:
@@ -99,7 +109,7 @@ def do_things(arguments):
         template = env.get_template(tmpfile)
         # print(kwargs)
         filecontent = template.render(**kwargs)
-        tree(filecontent)
+        tree(filecontent, quiet=_is_quiet)
         return
     # sharpshooter -j testjinja.tree replaceme=somefolder andme=another count=20
 
@@ -110,7 +120,27 @@ def do_things(arguments):
         filename = f"{name}.tree"
         with open(filename, "w") as f:
             f.write(
-                """
+                """# **********************************************************************
+# **********************************************************************
+#        _                          _                 _            
+#       | |                        | |               | |           
+#    ___| |__   __ _ _ __ _ __  ___| |__   ___   ___ | |_ ___ _ __ 
+#   / __| '_ \ / _` | '__| '_ \/ __| '_ \ / _ \ / _ \| __/ _ \ '__|
+#   \__ \ | | | (_| | |  | |_) \__ \ | | | (_) | (_) | ||  __/ |   
+#   |___/_| |_|\__,_|_|  | .__/|___/_| |_|\___/ \___/ \__\___|_|   
+#                        | |                                       
+#                        |_|                                       
+#
+#   This is a sharpshooter .tree file.
+#
+#   For more info visit: 
+#   https://github.com/byteface/sharpshooter/
+#
+#   sharpshooter (tree) is open sourced under a MIT License
+#
+# **********************************************************************
+# **********************************************************************
+
 +hello
     world.txt
 """
